@@ -166,23 +166,23 @@
 #include <structure.h>
 
 
- static FILE *file_asn1;		/* Pointer to file to parse */
+static FILE *file_asn1;			/* Pointer to file to parse */
 static asn1_retCode result_parse;	/* result of the parser
 					   algorithm */
-static node_asn *p_tree;		/* pointer to the root of the
+static ASN1_TYPE p_tree;		/* pointer to the root of the
 					   structure created by the
 					   parser*/
  static unsigned long lineNumber;	/* line number describing the
 					   parser position inside the
 					   file */
-static char lastToken[MAX_NAME_SIZE+1];	/* last token find in the file
+static char lastToken[ASN1_MAX_NAME_SIZE+1];	/* last token find in the file
 					   to parse before the 'parse
 					   error' */
 extern char _asn1_identifierMissing[];
 static const char *fileName;		/* file to parse */
 
-int _asn1_yyerror (char *);
-int _asn1_yylex(void);
+static int _asn1_yyerror (char *);
+static int _asn1_yylex(void);
 
 
 
@@ -209,8 +209,8 @@ typedef union YYSTYPE
 #line 62 "ASN1.y"
 {
   unsigned int constant;
-  char str[MAX_NAME_SIZE+1];
-  node_asn* node;
+  char str[ASN1_MAX_NAME_SIZE+1];
+  ASN1_TYPE node;
 }
 /* Line 187 of yacc.c.  */
 #line 217 "ASN1.c"
@@ -2356,20 +2356,22 @@ yyreturn:
 
 
 
-const char *key_word[]={"::=","OPTIONAL","INTEGER","SIZE","OCTET","STRING"
-                       ,"SEQUENCE","BIT","UNIVERSAL","PRIVATE","OPTIONAL"
-                       ,"DEFAULT","CHOICE","OF","OBJECT","IDENTIFIER"
-                       ,"BOOLEAN","TRUE","FALSE","APPLICATION","ANY","DEFINED"
-                       ,"SET","BY","EXPLICIT","IMPLICIT","DEFINITIONS","TAGS"
-                       ,"BEGIN","END","UTCTime","GeneralizedTime"
-                       ,"GeneralString","FROM","IMPORTS","NULL","ENUMERATED"};
-const int key_word_token[]={ASSIG,OPTIONAL,INTEGER,SIZE,OCTET,STRING
-                       ,SEQUENCE,BIT,UNIVERSAL,PRIVATE,OPTIONAL
-                       ,DEFAULT,CHOICE,OF,OBJECT,STR_IDENTIFIER
-                       ,BOOLEAN,TRUE,FALSE,APPLICATION,ANY,DEFINED
-                       ,SET,BY,EXPLICIT,IMPLICIT,DEFINITIONS,TAGS
-                       ,BEGIN,END,UTCTime,GeneralizedTime
-                       ,GeneralString,FROM,IMPORTS,TOKEN_NULL,ENUMERATED};
+static const char *key_word[] = {
+  "::=","OPTIONAL","INTEGER","SIZE","OCTET","STRING"
+  ,"SEQUENCE","BIT","UNIVERSAL","PRIVATE","OPTIONAL"
+  ,"DEFAULT","CHOICE","OF","OBJECT","IDENTIFIER"
+  ,"BOOLEAN","TRUE","FALSE","APPLICATION","ANY","DEFINED"
+  ,"SET","BY","EXPLICIT","IMPLICIT","DEFINITIONS","TAGS"
+  ,"BEGIN","END","UTCTime","GeneralizedTime"
+  ,"GeneralString","FROM","IMPORTS","NULL","ENUMERATED"};
+static const int key_word_token[] = {
+  ASSIG,OPTIONAL,INTEGER,SIZE,OCTET,STRING
+  ,SEQUENCE,BIT,UNIVERSAL,PRIVATE,OPTIONAL
+  ,DEFAULT,CHOICE,OF,OBJECT,STR_IDENTIFIER
+  ,BOOLEAN,TRUE,FALSE,APPLICATION,ANY,DEFINED
+  ,SET,BY,EXPLICIT,IMPLICIT,DEFINITIONS,TAGS
+  ,BEGIN,END,UTCTime,GeneralizedTime
+  ,GeneralString,FROM,IMPORTS,TOKEN_NULL,ENUMERATED};
 
 /*************************************************************/
 /*  Function: _asn1_yylex                                    */
@@ -2377,11 +2379,11 @@ const int key_word_token[]={ASSIG,OPTIONAL,INTEGER,SIZE,OCTET,STRING
 /*  Return: int                                              */
 /*    Token identifier or ASCII code or 0(zero: End Of File) */
 /*************************************************************/
-int
+static int
 _asn1_yylex()
 {
   int c,counter=0,k,lastc;
-  char string[MAX_NAME_SIZE+1]; /* will contain the next token */
+  char string[ASN1_MAX_NAME_SIZE+1]; /* will contain the next token */
 
   while(1)
     {
@@ -2428,7 +2430,7 @@ _asn1_yylex()
 	     c=='(' || c==')' || c=='[' || c==']' ||
 	     c=='{' || c=='}' || c==',' || c=='.'))
       {
-	if(counter>=MAX_NAME_SIZE){
+	if(counter>=ASN1_MAX_NAME_SIZE){
 	  result_parse=ASN1_NAME_TOO_LONG;
 	  return 0;
 	}
@@ -2467,7 +2469,7 @@ _asn1_yylex()
 /*    errorDescription: string that will contain the         */
 /*                      description.                         */
 /*************************************************************/
-void
+static void
 _asn1_create_errorDescription(int error,char *errorDescription)
 {
   switch(error){
@@ -2490,7 +2492,7 @@ _asn1_create_errorDescription(int error,char *errorDescription)
        strcat(errorDescription,":");
        _asn1_ltostr(lineNumber,errorDescription+strlen(fileName)+1);
        strcat(errorDescription,": name too long (more than ");
-       _asn1_ltostr(MAX_NAME_SIZE,errorDescription+strlen(errorDescription));
+       _asn1_ltostr(ASN1_MAX_NAME_SIZE,errorDescription+strlen(errorDescription));
        strcat(errorDescription," characters)");
     }
     break;
@@ -2538,7 +2540,7 @@ _asn1_create_errorDescription(int error,char *errorDescription)
   * is not defined.
   *
   * ASN1_NAME_TOO_LONG: In the file there is an identifier whith more
-  * than MAX_NAME_SIZE characters.
+  * than ASN1_MAX_NAME_SIZE characters.
   **/
 asn1_retCode
 asn1_parser2tree(const char *file_name, ASN1_TYPE *definitions,
@@ -2629,7 +2631,7 @@ asn1_parser2tree(const char *file_name, ASN1_TYPE *definitions,
   *   is not defined.
   *
   * ASN1_NAME_TOO_LONG: In the file there is an identifier whith more
-  *   than MAX_NAME_SIZE characters.
+  *   than ASN1_MAX_NAME_SIZE characters.
   **/
 int asn1_parser2array(const char *inputFileName,const char *outputFileName,
 		      const char *vectorName,char *errorDescription){
@@ -2736,7 +2738,7 @@ int asn1_parser2array(const char *inputFileName,const char *outputFileName,
 /*  Return: int                                              */
 /*                                                           */
 /*************************************************************/
-int _asn1_yyerror (char *s)
+static int _asn1_yyerror (char *s)
 {
   /* Sends the error description to the std_out */
 
@@ -2751,18 +2753,3 @@ int _asn1_yyerror (char *s)
   return 0;
 }
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
